@@ -1,15 +1,23 @@
 { package Catalyst::Action::SOAP::DocumentLiteral;
 
   use base qw/Catalyst::Action::SOAP/;
+  use constant NS_SOAP_ENV => "http://schemas.xmlsoap.org/soap/envelope/";
 
   sub execute {
       my $self = shift;
       my ( $controller, $c ) = @_;
-      $self->prepare_soap_helper($c);
-      $self->prepare_soap_xml_post($c);
+      $self->prepare_soap_helper($controller,$c);
+      $self->prepare_soap_xml_post($controller,$c);
       unless ($c->stash->{soap}->fault) {
           my $envelope = $c->stash->{soap}->parsed_envelope;
-          my ($body) = $envelope->getElementsByTagName('Body',0);
+          my $namespace = $c->stash->{soap}->namespace || NS_SOAP_ENV;
+          my ($body) = $envelope->getElementsByTagNameNS($namespace, 'Body');
+          my $operation = $self->name;
+          $c->stash->{soap}->operation_name($operation);
+          if ($controller->wsdlobj) {
+              $body = $c->stash->{soap}->arguments
+                ($controller->decoders->{$operation}->($body));
+          }
           $self->NEXT::execute($controller, $c, $body);
       }
   }
